// http://appnotes.etherkit.com/2016/04/10-milliwatt-si5351a-wspr-beacon/
// https://gist.github.com/NT7S/2b5555aa28622c1b3fcbc4d7c74ad926
// https://github.com/etherkit/JTEncode
// https://platformio.org/lib/show/495/Etherkit%20JTEncode

// Simple WSPR beacon for Arduino, with the Etherkit
// Si5351A Breakout Board, by Jason Milldrum NT7S.
//
// Transmit WSPR Type1 message by momentary pushing button to pin #9 
// LED on pin #13 is indicating time of transmission 

// Original code based on Feld Hell beacon for Arduino by Mark
// Vandewettering K6HX, adapted for the Si5351A by Robert
// Liesenfeld AK6L <ak6l@ak6l.org>.
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject
// to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
// ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
// CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

#include <si5351.h>
#include <JTEncode.h>
#include <rs_common.h>
#include <int.h>
#include <string.h>
#include "Wire.h"

// Mode defines
#define WSPR_TONE_SPACING       146          // ~1.46 Hz
#define WSPR_DELAY              683          // Delay value for WSPR

// Hardware defines
#define BUTTON                  9
#define LED_PIN                 13

// Class instantiation
Si5351 si5351;
JTEncode jtencode;

// Global variables; change first 4 lines according your needs
char message[] = "DJ7OO JN49";
char call[] = "DJ7OO";
char loc[] = "JN49";
uint8_t dbm = 27;
uint8_t tx_buffer[255];
uint8_t symbol_count;
uint16_t tone_delay, tone_spacing;
unsigned long freq;

////////////////////////////////////////////////////////////////
// Loop through the string, transmitting one character at a time.
void encode()
{
  uint8_t i;

// Reset the tone to the base frequency and turn on the output
  si5351.output_enable(SI5351_CLK0, 1);
  for(i = 0; i < symbol_count; i++)
  {
      si5351.set_freq((freq * 100) + (tx_buffer[i] * tone_spacing), SI5351_CLK0);
      delay(tone_delay);
  }

// Turn off the output
  si5351.output_enable(SI5351_CLK0, 0);
}

//////////////////////////////////////////////////////////
void set_tx_buffer()
{
// Clear out the transmit buffer
  memset(tx_buffer, 0, 255);
  jtencode.wspr_encode(call, loc, dbm, tx_buffer);
}

//////////////////////////////////////////////////////////////////////////
void setup()
{
  Serial.begin(9600);
  Serial.println("START");

// Initialize the Si5351
// si5351.init(SI5351_CRYSTAL_LOAD_8PF,0, 97000);         // 25MHz osc; with frq corr. value
// si5351.init(SI5351_CRYSTAL_LOAD_8PF,0, 0);             // 25MHz osc; without frq corr. value
   si5351.init(SI5351_CRYSTAL_LOAD_8PF, 27000000,240000); // 27MHz osc; with frq corr. value

// Use the Arduino's on-board LED as a keying indicator.
  pinMode(LED_PIN, OUTPUT);
  digitalWrite(LED_PIN, LOW);

// Use a button connected to pin 9 as a transmit trigger
  pinMode(BUTTON, INPUT_PULLUP);

// Set the proper frequency, tone spacing, symbol count, and
// tone delay depending on mode

// freq =   475700; // 630m-Band   474200 +1400-1600
// freq =  3594100; //  80m-Band  3592600 +1400-1600
// freq =  7040100; //  40m-Band  7038600 +1400-1600
// freq = 10140200; //  30m-Band 10138700 +1400-1600
// freq = 14097100; //  20m-Band 14095600 +1400-1600
   freq = 28126100; //  10m-Band 28124600 +1400-1600         
        
    symbol_count = WSPR_SYMBOL_COUNT; // From the library defines
    tone_spacing = WSPR_TONE_SPACING;
    tone_delay = WSPR_DELAY;

// Set CLK0 output
  si5351.drive_strength(SI5351_CLK0, SI5351_DRIVE_2MA); // Set for max power if desired
  si5351.output_enable(SI5351_CLK0, 0);                 // Disable the clock initially

// Encode the message in the transmit buffer
// This is RAM intensive and should be done separately from other subroutines
  set_tx_buffer();
}

//////////////////////////////////////////////////////////////////////////////////
void loop()
{
// Debounce the button and trigger TX on push
  if(digitalRead(BUTTON) == LOW)
  {
   delay(50);   // delay to debounce
   if (digitalRead(BUTTON) == LOW)
    {
     Serial.println("Start sending WSPR");
     digitalWrite(LED_PIN, HIGH); 
     encode();
     Serial.println("  End sending WSPR");     
     digitalWrite(LED_PIN, LOW);
     delay(1000);      
    }
  }    
}

